#include <string>

#include <json/json.h>

//Internal functions used in ILGTP to monitor the state of the client and the returned value after a request
bool checkStatusFn(const std::string& toParse, unsigned short VERBOSE_LEVEL){
	Json::Value root;
	Json::Reader reader;
	if(!reader.parse(toParse, root)){
		VERBOSE_3<<"ERROR: Invalid message"<<std::endl;
		VERBOSE_3<<toParse<<std::endl;
		return false;
	}
	
	std::vector<std::string> members(root.getMemberNames());
	if(members.size()!=1 || root.isNull()){
		VERBOSE_3<<"ERROR: Invalid format of the status message"<<std::endl;
		return false;
	}
	
	const Json::Value status = root[members[0]]["status"];
	if(status.isNull() || status.empty()){
		VERBOSE_3<<"ERROR: Impossible to get status from the message"<<std::endl;
		return false;
	}
	
	if(!status.isString()){
		VERBOSE_3<<"ERROR: The status is not in a string format, invalid message"<<std::endl;
		return false;
	}
	
	return(status.asString()=="OK");
}

//The error message for this function is generated in the function above.
#define checkStatus(result, VERBOSE_LEVEL) \
	{if(!checkStatusFn(result.second, VERBOSE_LEVEL)) {\
                return false;\
        }}

#define checkClient() \
	{if(!client.isConnected()) {\
		VERBOSE_1 << "ERROR: Not connected to MsgServer" << std::endl;\
		return false;\
	}}

#define clearMessageStack()\
	{while(client.isMessageWaiting()){\
		client.getMessage();\
	}}
