#if !defined HATP_REQUESTLEXER_HH_
#define HATP_REQUESTLEXER_HH_

#include <boost/regex.hpp>

// ---------- String tools ----------

static std::string& str_replace(const std::string &search, const std::string &replace, std::string &subject) {
    std::string buffer;
    int sealeng = search.length();
    int strleng = subject.length();
    if (sealeng==0) return subject;
    for(int i=0, j=0; i<strleng; j=0 ) {
        while (i+j<strleng && j<sealeng && subject[i+j]==search[j]) j++;
        if (j==sealeng) {buffer.append(replace); i+=sealeng;}
        else buffer.append( &subject[i++], 1);
    }
    subject = buffer;
    return subject;
}

static std::string& removeFormatting(std::string &s) {
	return str_replace(" ", "", s);
}

// ---------- Default parsing ----------

static const std::string HSPACE		= "[[:space:]]";
static const std::string HWORD		= "([[:word:]]+)";
static const std::string HLPAREN		= "[(]";
static const std::string HRPAREN		= "[)]";
static const std::string HLLIST		= "[(.][[.period.]]" + HSPACE;
static const std::string HRLIST		= "[[.period.]][)]";
static const std::string HCLIST		= "(([[:word:]]+\\s)*)";
static const std::string HLIST		= HLLIST + HCLIST + HRLIST;

static const std::string HLSCOPE		= "[{]";
static const std::string HRSCOPE		= "[}]";
static const std::string HLBRAC		= "[[.left-square-bracket.]]";
static const std::string HRBRAC		= "[[.right-square-bracket.]]";
static const std::string HJSEP		= "[[.colon.]]";
static const std::string HCOMMA		= "[[.comma.]]";
static const std::string HQUOTE		= "[[.quotation-mark.]]";
static const std::string HSTRING		= HQUOTE + HWORD + HQUOTE;
static const std::string HNUMBER		= "([[:digit:]]+)";
static const std::string HSLIST		= HLBRAC + "((("+HSTRING+")|(" + HSTRING + HCOMMA + "))*)" + HRBRAC;

// ---------- Ontology result parsing ----------

static const std::string HK_ONTOREP	= HQUOTE + "ONTO-REP" + HQUOTE;
static const std::string HK_VALUE	= HQUOTE + "VALUE" + HQUOTE;

static const std::string ONTOVAL		= HK_VALUE + HJSEP + HSLIST;

static const std::string ONTORES		= HLSCOPE + HK_ONTOREP + HJSEP + HLSCOPE + ONTOVAL + HRSCOPE + HRSCOPE;

// ---------- Ontology copy result parsing ----------

static const std::string HK_ONTOALLREP	= HQUOTE + "ONTO-REP-ALL" + HQUOTE;
static const std::string HK_OBJECTS		= HQUOTE + "OBJECTS" + HQUOTE;

static const std::string ONTOVALSCOPE	= HLSCOPE + HK_VALUE + HJSEP + HLBRAC + ".*?" + HRBRAC + HRSCOPE;
static const std::string VALUE		= "[[:word:][.period.]_]+";

static const std::string ONTOALLRES		= HLSCOPE + HK_ONTOALLREP + HJSEP + HLSCOPE + HK_OBJECTS + HJSEP + HLBRAC + "(.*)" + HRBRAC + HRSCOPE + HRSCOPE;


// ---------- REGULAR EXPRESSION  ----------

static const boost::regex ONTOResult(ONTORES);
static const boost::regex ONTOALLResult(ONTOALLRES);
static const boost::regex ONTOValueScope(ONTOVALSCOPE);
static const boost::regex ONTOValue(VALUE);
#endif
